ZeedhiFields.directive("zhIcon", function(ZhSprite, ZhLogger, templateManager) {
	return {
		restrict: 'A',
		link: function(scope, element, attrs) {
			function injectSVG(iconName) {
				var icon = ZhSprite.getIcon(iconName);

				if (!icon) {
					ZhLogger.warning('Icon "' + iconName + '" not found on sprite!');
				} else {
					var svgElement = ZhSprite.getSprite().querySelector('svg');
					var svgChild = angular.element('<svg width="' + svgElement.getAttribute('width') + '" height="' + svgElement.getAttribute('height') + '" xmlns="http://www.w3.org/2000/svg" version="1.1"></svg>');

					if (!element.hasClass('zh-icon-no-border')) {
						var svgCircle = ZhSprite.getIcon("border");
						svgChild.append($(svgCircle).clone());
					}

					svgChild.append($(icon).clone());
					element.append(svgChild);
				}

				var displayValue = 'block';
				if (element.hasClass("zh-icon-inline")) {
					displayValue = 'inline';
				} else if (element.hasClass("zh-icon-inline-block")) {
					displayValue = 'inline-block';
				}
				element.css('display', displayValue);
			}

			function clearIconAndStyles(element) {
				if (element.hasIcon || element.children().length) {
					element.attr("style", "");
					element.empty();
					element.hasIcon = false;
				}
			}

			function clearClasses(element) {
				var classList = element[0].classList.length ? element[0].classList.toString().split(" ") : [];

				if (classList.length) {
					classList.forEach(function(str) {
						if (str.substr(0, 7) === "zh-icon") {
							element.removeClass(str);
						}
					});
				}
			}

			function processIcon() {
				var iconString = attrs.zhIcon;
				var iconName, otherClasses = "";

				clearClasses(element);
				// Filters the icon string in search of the icon name
				iconString.split(" ").forEach(function(classStr) {
					switch (classStr) {
						case "icon-black":
							otherClasses += " zh-icon-color-black";
							break;
						case "icon-white":
							otherClasses += " zh-icon-color-white";
							break;
						case "icon-main":
							otherClasses += " zh-icon-color-main";
							break;
						case "no-border":
							otherClasses += " zh-icon-no-border";
							break;
						case "inline":
							otherClasses += " zh-icon-inline";
							break;
						case "inline-block":
							otherClasses += " zh-icon-inline-block";
							break;
						default:
							iconName = classStr;
					}
				});

				if (iconName) {
					element.addClass('zh-icon zh-icon-' + iconName + otherClasses);
					clearIconAndStyles(element);

					var spriteUrl = getSpriteUrl();
					if (spriteUrl) {
						element.hasIcon = true;
						element.css('backgroundImage', 'none');
						element.bkgImageUrl = spriteUrl;
						ZhSprite.loadSprite(spriteUrl).then(function() {
							injectSVG(iconName);
						});
					}
				}
			}

			function getSpriteUrl() {
				var backgroundUrl = ZhSprite.getSpriteUrl();
				if (!backgroundUrl) {
					backgroundUrl = templateManager.project.iconSpriteFile;
					ZhSprite.setSpriteUrl(backgroundUrl);
				}
				return backgroundUrl;
			}

			attrs.$observe("zhIcon", function(newValue, oldValue) {
				if (newValue !== oldValue) {
					processIcon();
				}
			});
		}
	};
});